package ch.zhaw.gartenverwaltung.io;

import ch.zhaw.gartenverwaltung.types.HardinessZone;
import ch.zhaw.gartenverwaltung.types.Task;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;
import com.fasterxml.jackson.datatype.jsr310.deser.LocalDateDeserializer;
import com.fasterxml.jackson.datatype.jsr310.deser.YearMonthDeserializer;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.time.LocalDate;
import java.time.YearMonth;
import java.time.format.DateTimeFormatter;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class JsonTaskDatabase implements TaskDatabase{
    private final URL dataSource = getClass().getResource("taskdb.json");

    private Map<Long, Task> taskMap = Collections.emptyMap();

    private final static JavaTimeModule timeModule = new JavaTimeModule();
    static {
        DateTimeFormatter dateFormat = DateTimeFormatter.ofPattern("yy-MM");
        YearMonthDeserializer dateDeserializer = new YearMonthDeserializer(dateFormat);
        timeModule.addDeserializer(YearMonth.class, dateDeserializer);
    }

    @Override
    public List<Task> getTaskList(LocalDate start, LocalDate end) throws IOException{

        if(taskMap.isEmpty()) {
            loadTaskListFromFile();
        }
        return taskMap.values().stream().filter(task -> task.isInTimePeriode(start, end)).toList();
    }

    @Override
    public void saveTask(Task task) throws IOException {
        ObjectMapper mapper = new ObjectMapper();
        if(dataSource != null) {
            mapper.writeValue(new File(dataSource.getFile()), task);
        }
    }

    @Override
    public void removeTask(Task task) throws IOException {
        if(!taskMap.isEmpty() && taskMap.containsKey(task.getId())){
            taskMap.remove(task.getId());
        }
    }

    private void writeTaskListToFile() throws IOException {
        ObjectMapper mapper = new ObjectMapper();
        mapper.registerModule(timeModule);

        mapper.writeValue(new File(dataSource.getFile()), taskMap);
    }


    private void loadTaskListFromFile() throws IOException {
        if (dataSource != null) {
            ObjectMapper mapper = new ObjectMapper();
            mapper.registerModule(timeModule);

            List<Task> result;
            result = mapper.readerForListOf(Task.class).readValue(dataSource);

            taskMap = result.stream()
                    .collect(HashMap::new,
                        (res, task) -> res.put(task.getId(), task),
                        (existing, replacement) -> {});
            System.out.println(taskMap);
        }
    }

    //Test main method
    public static void main(String[] args) {
        LocalDate date = LocalDate.now();
        LocalDate yesterday = LocalDate.now();
        yesterday = yesterday.minusDays(1);
        Task testTask = new Task("water", "apply water", date);
        JsonTaskDatabase jsonTaskDatabase = new JsonTaskDatabase();
        JsonPlantDatabase jsonPlantDatabase = new JsonPlantDatabase();

        try {
            //test load file
            jsonPlantDatabase.getPlantList(HardinessZone.ZONE_8A);
            jsonTaskDatabase.loadTaskListFromFile();

        } catch (Exception e){
            System.out.println("Task load failed!: " + e.getMessage());
        }
    }
}
